import requests
import json

from .auth import AuthClient


class ConverterClient:

    def __init__(self, converter_url):
        self.converter_url = converter_url

    def start(self, job_config):
        """Schedule a new converter job

        :param job_config: json object of job configuration
        :return: job_id
        """
        keycloak_token = AuthClient.fetch_personal_token()
        converter_response = requests.post(
            f'{self.converter_url}/jobs',
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'},
            data=json.dumps(job_config)
        )
        converter_response.raise_for_status()
        return converter_response


    def start_simulation(self, job_config):
        """Start a simulated converter job

        Useful for testing job configurations or diagnosing pseudonymization issues.

        :param job_config: json object of job configuration
        :return: job_id
        """
        keycloak_token = AuthClient.fetch_personal_token()
        converter_response = requests.post(
            f'{self.converter_url}/jobs/simulation',
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'},
            data=json.dumps(job_config)
        )
        converter_response.raise_for_status()
        return converter_response


    def get_job_summary(self, job_id):
        """Retrieve the execution summary for a specific converter job

        :param job_id: job id returned after starting converter job using start()
        :return: job summary
        """
        keycloak_token = AuthClient.fetch_personal_token()

        job_summary = requests.get(
            f'{self.converter_url}/jobs/{job_id}/execution-summary',
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'}
        )
        job_summary.raise_for_status()
        return job_summary


    def stop_job(self, job_id):
        """Stop a specific converter job

        :param job_id:
        :return:
        """
        keycloak_token = AuthClient.fetch_personal_token()

        job_status = requests.post(
            f'{self.converter_url}/jobs/{job_id}/stop',
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'}
        )
        job_status.raise_for_status()
        return job_status


    def get_pseudo_report(self, job_id):
        """Get a report with details about how pseudonymization is being applied for a specific job

        The report includes:
        * mathced pseudo rules
        * unmatched pseudo rules
        * pseudo rule to field match map - rule names and their corresponding matches
        * field matches to pseudo rule map - field names and the corresponding pseudo rule that covers this field
        * metrics
        * textual schema hierarchy that illustrates how the pseudo rules are being applied

        :param job_id: job id returned after starting converter job using start()
        :return: job summary
        """
        keycloak_token = AuthClient.fetch_personal_token()

        pseudo_report = requests.get(
            f'{self.converter_url}/jobs/{job_id}/reports/pseudo',
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'}
        )
        pseudo_report.raise_for_status()
        return pseudo_report


    def get_pseudo_schema(self, job_id):
        """Get hierarchical schema representation that details how pseudo rules are being applied

        This is a smaller version get_pseudo_report

        :param job_id: job id returned after starting converter job using start()
        :return: job summary
        """
        keycloak_token = AuthClient.fetch_personal_token()

        pseudo_report = requests.get(
            f'{self.converter_url}/jobs/{job_id}/reports/pseudo-schema-hierarchy',
            headers={'Authorization': 'Bearer %s' % keycloak_token, 'Content-type': 'application/json'}
        )
        pseudo_report.raise_for_status()
        return pseudo_report



import os
from datetime import datetime, timedelta
from functools import partial

import google.auth
import requests
from google.oauth2.credentials import Credentials
from IPython.display import HTML, display

from jupyterhub.services.auth import HubAuth


class AuthClient:
    """
    A client class that connects to the AuthHandler to retrieve user and auth state info
    """

    @staticmethod
    def fetch_local_user():
        if AuthClient.is_oidc_token():
            response = requests.post(
                os.environ["OIDC_TOKEN_EXCHANGE_URL"],
                data={
                    "subject_token": os.environ["OIDC_TOKEN"],
                    "grant_type": "urn:ietf:params:oauth:grant-type:token-exchange",
                    "requested_token_type": "urn:ietf:params:oauth:token-type:access_token",
                    "requested_issuer": "google",
                    "client_id": "onyxia",
                },
            )
            if response.status_code == 200:
                # The 'exp' field is not provided in the token exchange response.
                # 'exp' is calculated by adding 'expires_in' seconds to the current UTC time,
                # resulting in a POSIX timestamp representing the expiration time.
                exp = (
                    datetime.utcnow() + timedelta(seconds=response.json()["expires_in"])
                ).timestamp()
                return {
                    "access_token": os.environ["OIDC_TOKEN"],
                    "exchanged_tokens": {
                        "google": {
                            "access_token": response.json()["access_token"],
                            "exp": exp,
                        }
                    },
                }
            else:
                raise AuthError
        else:
            # Helps getting the correct ssl configs
            hub = HubAuth()
            response = requests.get(
                os.environ["LOCAL_USER_PATH"],
                headers={"Authorization": "token %s" % hub.api_token},
                cert=(hub.certfile, hub.keyfile),
                verify=hub.client_ca,
                allow_redirects=False,
            )
            if response.status_code == 200:
                return response.json()
            else:
                raise AuthError

    @staticmethod
    def fetch_personal_token():
        try:
            return AuthClient.fetch_local_user()["access_token"]
        except AuthError as err:
            err.print_warning()

    @staticmethod
    def fetch_google_token():
        try:
            return AuthClient.fetch_local_user()["exchanged_tokens"]["google"][
                "access_token"
            ]
        except AuthError as err:
            err.print_warning()

    @staticmethod
    def fetch_google_credentials():
        if AuthClient.is_ready():
            try:
                local_user = AuthClient.fetch_local_user()
                credentials = Credentials(
                    token=local_user["exchanged_tokens"]["google"]["access_token"],
                    expiry=datetime.utcfromtimestamp(
                        local_user["exchanged_tokens"]["google"]["exp"]
                    ),
                    token_uri="https://oauth2.googleapis.com/token",
                )
            except AuthError as err:
                err.print_warning()

            def _refresh(self, _request):
                try:
                    user = AuthClient.fetch_local_user()
                    self.token = user["exchanged_tokens"]["google"]["access_token"]
                    self.expiry = datetime.utcfromtimestamp(
                        user["exchanged_tokens"]["google"]["exp"]
                    )
                except AuthError as err:
                    err.print_warning()

            credentials.refresh = partial(_refresh, credentials)
            return credentials
        else:
            # Fetch credentials from Google Cloud SDK
            credentials, _ = google.auth.default()
            return credentials

    @staticmethod
    def is_ready():
        return "LOCAL_USER_PATH" in os.environ or "OIDC_TOKEN" in os.environ

    @staticmethod
    def is_oidc_token():
        return "OIDC_TOKEN" in os.environ


class AuthError(Exception):
    """This exception class is used when the communication with the custom auth handler fails.
    This is normally due to stale auth session."""

    def print_warning(self):
        display(
            HTML(
                'Your session has timed out. Please <a href="/hub/login">log in</a> to continue.'
            )
        )



__version__ = "1.8.5"

from .auth import AuthClient
from .backports import details, show
from .collector import CollectorClient
from .converter import ConverterClient
from .doctor import Doctor
from .files import FileClient
from .git import repo_root_dir
from .guardian import GuardianClient
from .jupyterhub import generate_api_token
from .pandas import read_pandas, write_pandas
from .pubsub import trigger_source_data_processing
